import 'dart:async';
import 'dart:convert';
import 'dart:html';

typedef OnMessageCallback = void Function(dynamic msg);
typedef OnCloseCallback = void Function(int code, String reason);
typedef OnOpenCallback = void Function();

enum ConnectionStatus {
  connecting,
  connected,
  closed,
}

class BaseWebSocket {
  String url;
  WebSocket socket;
  OnOpenCallback onOpen;
  OnMessageCallback onMessage;
  OnCloseCallback onClose;
  Duration ping;

  BaseWebSocket(this.url, {this.ping = const Duration(seconds: 5)}) {
    url = url.replaceAll('https:', 'wss:');
  }
  ConnectionStatus connectionStatus;
  Timer _t;

  void connect() {
    try {
      connectionStatus = ConnectionStatus.connecting;
      socket = WebSocket(url);
      socket.onOpen.listen((e) {
        onOpen();
        _t = Timer.periodic(ping, (t) {
          socket.send('');
        });
        connectionStatus = ConnectionStatus.connected;
      });

      socket.onMessage.listen((e) {
        print("received: ${e?.data}");
        onMessage(e?.data);
      });

      socket.onClose.listen((e) {
        _t?.cancel();
        connectionStatus = ConnectionStatus.closed;
        onClose(e.code, e.reason);
      });
      socket.onError.listen((event) {
        _t?.cancel();
      });
    } catch (e) {
      _t?.cancel();
      onClose(500, e.toString());
    }
  }

  void send(dynamic data) {
    if (connectionStatus == ConnectionStatus.closed) {
      connect();
    }
    if (socket != null && socket.readyState == WebSocket.OPEN) {
      socket.send(data);
      print('send: $data');
    } else {
      print('WebSocket not connected, message $data not sent');
    }
  }

  void emit(String event, dynamic data) {
    if (connectionStatus == ConnectionStatus.closed) {
      connect();
    }
    if (socket != null && socket.readyState == WebSocket.OPEN) {
      socket.send(json.encode({'event': event, 'data': data}).toString());
      print('send: $data');
    } else {
      print('WebSocket not connected, message $data not sent');
    }
  }

  void close() {
    if (socket != null) socket.close();
  }
}
