import 'dart:convert';
import 'dart:io';
import 'dart:async';

typedef OnMessageCallback = void Function(dynamic msg);
typedef OnCloseCallback = void Function(int code, String reason);
typedef OnOpenCallback = void Function();

enum ConnectionStatus {
  connecting,
  connected,
  closed,
}

class BaseWebSocket {
  String url;
  WebSocket socket;
  OnOpenCallback onOpen;
  OnMessageCallback onMessage;
  OnCloseCallback onClose;
  BaseWebSocket(this.url, {this.ping = const Duration(seconds: 5)});
  Duration ping;

  ConnectionStatus connectionStatus;

  Future connect() async {
    try {
      connectionStatus = ConnectionStatus.connecting;
      socket = await WebSocket.connect(url);

      socket.pingInterval = ping;

      onOpen();
      connectionStatus = ConnectionStatus.connected;
      socket.listen((data) {
        print("received: $data");
        onMessage(data);
      }, onDone: () {
        connectionStatus = ConnectionStatus.closed;
        onClose(socket.closeCode, socket.closeReason);
      }, onError: (e) {
        onClose(500, e.toString());
      });

      return;
    } catch (e) {
      onClose(500, e.toString());

      return;
    }
  }

  void send(dynamic data) async {
    if (connectionStatus == ConnectionStatus.closed) {
      await connect();
    }

    if (socket != null) {
      socket.add(data);
      print('send: $data');
    }
  }

  Future<void> emit(String event, dynamic data) async {
    if (connectionStatus == ConnectionStatus.closed) {
      await connect();
    }

    if (socket != null) {
      socket.add(json.encode({'event': event, 'data': data}).toString());
      print('send: $data');
    }
  }

  void close() {
    if (socket != null) {
      socket.close();
    }
  }

  // Future<WebSocket> _connectForSelfSignedCert(url) async {
  //   try {
  //     var r = Random();
  //     var key = base64.encode(List<int>.generate(8, (_) => r.nextInt(255)));
  //     var client = HttpClient(context: SecurityContext());
  //     client.badCertificateCallback =
  //         (X509Certificate cert, String host, int port) {
  //       print(
  //           'BaseWebSocket: Allow self-signed certificate => $host:$port. ');
  //       return true;
  //     };

  //     var request =
  //         await client.getUrl(Uri.parse(url)); // form the correct url here
  //     request.headers.add('Connection', 'Upgrade');
  //     request.headers.add('Upgrade', 'websocket');
  //     request.headers.add(
  //         'Sec-WebSocket-Version', '13'); // insert the correct version here
  //     request.headers.add('Sec-WebSocket-Key', key.toLowerCase());

  //     var response = await request.close();
  //     var socket = await response.detachSocket();
  //     var webSocket = WebSocket.fromUpgradedSocket(
  //       socket,
  //       protocol: 'signaling',
  //       serverSide: false,
  //     );

  //     return webSocket;
  //   } catch (e) {
  //     rethrow;
  //   }
  // }
}
